searchData={"items":[{"type":"module","title":"ProofOfReserves","doc":"Documentation for `ProofOfReserves`.","ref":"ProofOfReserves.html"},{"type":"function","title":"ProofOfReserves.build_liabilities_tree/3","doc":"build_liabilities_tree builds a Merkle Sum Tree from a list of liabilities.\nFirst, we split the liabilities to mask account balances and ensure the\nliability count is a power of 2, then we shuffle them.\nThen, we convert the liabilities to nodes and build the Merkle Sum Tree.","ref":"ProofOfReserves.html#build_liabilities_tree/3"},{"type":"function","title":"ProofOfReserves.find_balances_for_accounts/3","doc":"find_balances_for_accounts finds all leaves that belong to a particular\naccount using the attestation_key","ref":"ProofOfReserves.html#find_balances_for_accounts/3"},{"type":"function","title":"ProofOfReserves.get_tree_root/1","doc":"get_tree_root returns the root node of a Merkle Sum Tree.","ref":"ProofOfReserves.html#get_tree_root/1"},{"type":"function","title":"ProofOfReserves.serialize_liabilities/2","doc":"serialize_liabilities formats the Proof of Liabilities into a String.","ref":"ProofOfReserves.html#serialize_liabilities/2"},{"type":"function","title":"ProofOfReserves.shuffle_liabilities/1","doc":"shuffle_liabilities shuffles the liabilities using cryptographic randomness.\nIt does this by zipping each liability with a random number, sorting by the random number,\nthen extracting the liability.","ref":"ProofOfReserves.html#shuffle_liabilities/1"},{"type":"function","title":"ProofOfReserves.split_liabilities/2","doc":"split_liabilities divides the liabilities according to the following rules:\n1. split all liabilities at least once (unless the liability is 1 sat).\n2. All leaves must be below the threshold.\n3. Divide the liabilities until a power of two is reached. If this is impossible,\n   we add zero-amount liabilities to reach the next power of two.","ref":"ProofOfReserves.html#split_liabilities/2"},{"type":"module","title":"ProofOfReserves.Liabilities","doc":"Liabilities is a module that is used to calculate a Proof of Liabilities.","ref":"ProofOfReserves.Liabilities.html"},{"type":"function","title":"ProofOfReserves.Liabilities.dummy_liability/0","doc":"dummy_liability returns a dummy liability","ref":"ProofOfReserves.Liabilities.html#dummy_liability/0"},{"type":"function","title":"ProofOfReserves.Liabilities.fake_liability/1","doc":"","ref":"ProofOfReserves.Liabilities.html#fake_liability/1"},{"type":"function","title":"ProofOfReserves.Liabilities.fake_liability_account_subkey/0","doc":"","ref":"ProofOfReserves.Liabilities.html#fake_liability_account_subkey/0"},{"type":"function","title":"ProofOfReserves.Liabilities.liability_to_node/3","doc":"liability_to_node converts a liability to a node in the Merkle Sum Tree.","ref":"ProofOfReserves.Liabilities.html#liability_to_node/3"},{"type":"function","title":"ProofOfReserves.Liabilities.parse_liabilities/1","doc":"parse_liabilities parses the liabilities from a stream","ref":"ProofOfReserves.Liabilities.html#parse_liabilities/1"},{"type":"function","title":"ProofOfReserves.Liabilities.serialize_liabilities/2","doc":"serialize_liabilities serializes the liabilities into a string","ref":"ProofOfReserves.Liabilities.html#serialize_liabilities/2"},{"type":"function","title":"ProofOfReserves.Liabilities.split_liabilities_to_power_of_two/1","doc":"split_liabilities_to_power_of_two divides the liabilities until a power of two is reached.\nnote: This logic attempts to use splitting to get to a power of two.\nbut if we can't get there, we add zero-amount liabilities. It may be\nsimpler to just add zero-amount liabilities until we reach the next power of two.","ref":"ProofOfReserves.Liabilities.html#split_liabilities_to_power_of_two/1"},{"type":"function","title":"ProofOfReserves.Liabilities.split_liability/1","doc":"split_liability divides a liability into 2 liabilities whose amounts sum\nto the amount of the original liability.\nThis function should never be called with amount <= 1 sat","ref":"ProofOfReserves.Liabilities.html#split_liability/1"},{"type":"function","title":"ProofOfReserves.Liabilities.split_liability_below_threshold/2","doc":"","ref":"ProofOfReserves.Liabilities.html#split_liability_below_threshold/2"},{"type":"function","title":"ProofOfReserves.Liabilities.sum_liabilities/1","doc":"sum_liabilities calculates the sum of a list of liabilities.","ref":"ProofOfReserves.Liabilities.html#sum_liabilities/1"},{"type":"module","title":"ProofOfReserves.Liability","doc":"A liability is a financial obligation that a company has to a user account.\nLiabilities are calculated on an account basis.\n\nBitMEX calls the account_subkey an account_nonce, but we renamed it account_subkey\nsince it is not in fact a nonce, and is long-lived and derived from the account's\nroot key, email, and account_id.","ref":"ProofOfReserves.Liability.html"},{"type":"function","title":"ProofOfReserves.Liability.new/3","doc":"new returns a new Liability","ref":"ProofOfReserves.Liability.html#new/3"},{"type":"type","title":"ProofOfReserves.Liability.t/0","doc":"","ref":"ProofOfReserves.Liability.html#t:t/0"},{"type":"module","title":"ProofOfReserves.MerkleSumTree","doc":"MerkleSumTree is a module that represents a Merkle Sum Tree. It is used to calculate a Proof of Liabilities.\nIn this module, a Node struct can represent a leaf or a branch. But when a variable is named leaves, it refers to the lowest level of the tree.","ref":"ProofOfReserves.MerkleSumTree.html"},{"type":"function","title":"ProofOfReserves.MerkleSumTree.build_merkle_tree/1","doc":"build_merkle_tree builds a Merkle Sum Tree from a list of leaves.\nReturns an error if the number of leaves is not a power of 2.","ref":"ProofOfReserves.MerkleSumTree.html#build_merkle_tree/1"},{"type":"function","title":"ProofOfReserves.MerkleSumTree.get_leaves/1","doc":"get_leaves returns the leaves (the lowest level) of a tree.","ref":"ProofOfReserves.MerkleSumTree.html#get_leaves/1"},{"type":"function","title":"ProofOfReserves.MerkleSumTree.get_tree_root/1","doc":"get_tree_root returns the root of a tree if it is complete.","ref":"ProofOfReserves.MerkleSumTree.html#get_tree_root/1"},{"type":"function","title":"ProofOfReserves.MerkleSumTree.merkleize_one_level/1","doc":"merkleize_one_level takes a list of nodes and combines them into a new list of nodes which will be half the size.","ref":"ProofOfReserves.MerkleSumTree.html#merkleize_one_level/1"},{"type":"function","title":"ProofOfReserves.MerkleSumTree.parse_tree/1","doc":"parse_tree parses a stream of serialized Node strings into a tree.","ref":"ProofOfReserves.MerkleSumTree.html#parse_tree/1"},{"type":"function","title":"ProofOfReserves.MerkleSumTree.serialize_tree/1","doc":"serialize_tree serializes a tree into a list of strings, where each string represents a node.\nThis is a breadth first serialization, where each level is serialized in order.","ref":"ProofOfReserves.MerkleSumTree.html#serialize_tree/1"},{"type":"function","title":"ProofOfReserves.MerkleSumTree.verify_tree?/1","doc":"verify_tree? rebuilds the entire tree and compares the root and the height of the tree.","ref":"ProofOfReserves.MerkleSumTree.html#verify_tree?/1"},{"type":"type","title":"ProofOfReserves.MerkleSumTree.tree/0","doc":"","ref":"ProofOfReserves.MerkleSumTree.html#t:tree/0"},{"type":"module","title":"ProofOfReserves.MerkleSumTree.Node","doc":"Node represents a node in a Merkle Sum Tree. It has a hash and a value.","ref":"ProofOfReserves.MerkleSumTree.Node.html"},{"type":"function","title":"ProofOfReserves.MerkleSumTree.Node.merkleize_nodes/1","doc":"merkleize_nodes takes two nodes and combines them into a new node.","ref":"ProofOfReserves.MerkleSumTree.Node.html#merkleize_nodes/1"},{"type":"function","title":"ProofOfReserves.MerkleSumTree.Node.new/2","doc":"new creates a new node with the given hash and value.","ref":"ProofOfReserves.MerkleSumTree.Node.html#new/2"},{"type":"function","title":"ProofOfReserves.MerkleSumTree.Node.parse_node/1","doc":"parse_node parses a string of \"hash,value\" into a Node.\nvalue is trimmed of leading and trailing whitespace before\nbeing converted to an integer.","ref":"ProofOfReserves.MerkleSumTree.Node.html#parse_node/1"},{"type":"function","title":"ProofOfReserves.MerkleSumTree.Node.serialize_node/1","doc":"serialize_node serializes a node into a string of \"hash,value\".","ref":"ProofOfReserves.MerkleSumTree.Node.html#serialize_node/1"},{"type":"type","title":"ProofOfReserves.MerkleSumTree.Node.t/0","doc":"","ref":"ProofOfReserves.MerkleSumTree.Node.html#t:t/0"},{"type":"module","title":"ProofOfReserves.Util","doc":"","ref":"ProofOfReserves.Util.html"},{"type":"function","title":"ProofOfReserves.Util.abbr_hash/1","doc":"Replaces all but the first and last 2 bytes of a hash with \"...\".","ref":"ProofOfReserves.Util.html#abbr_hash/1"},{"type":"function","title":"ProofOfReserves.Util.base32_to_int!/1","doc":"base32_to_int! converts a base32 string to an integer.","ref":"ProofOfReserves.Util.html#base32_to_int!/1"},{"type":"function","title":"ProofOfReserves.Util.bin_to_hex!/1","doc":"bin_to_hex! converts a binary to a lowercase hex string.","ref":"ProofOfReserves.Util.html#bin_to_hex!/1"},{"type":"function","title":"ProofOfReserves.Util.calculate_account_subkey/3","doc":"calculate_account_subkey generates the subkey for an account.\naccount_subkey = sha256(account_key || email || account_id)\naccount_key is a 32-byte binary, not a hex String.\nemail is a String, which is UTF-8 encoded and thus needs no\nspecial handling before hashing","ref":"ProofOfReserves.Util.html#calculate_account_subkey/3"},{"type":"function","title":"ProofOfReserves.Util.calculate_attestation_key/3","doc":"calculate_attestation_key generates the attestation key for the account in a specific attestation.\nattestation_key = sha256(account_subkey || block_height || account_id)\nblock_height and account_id must be 8-byte ints.","ref":"ProofOfReserves.Util.html#calculate_attestation_key/3"},{"type":"function","title":"ProofOfReserves.Util.crypto_rand_int/0","doc":"crypto_rand_int generates a random 64-bit unsigned integer.","ref":"ProofOfReserves.Util.html#crypto_rand_int/0"},{"type":"function","title":"ProofOfReserves.Util.hex_to_bin!/1","doc":"hex_to_bin! converts a lowercase hex string to a binary.","ref":"ProofOfReserves.Util.html#hex_to_bin!/1"},{"type":"function","title":"ProofOfReserves.Util.int_to_little/2","doc":"int_to_little converts an integer to a little-endian binary.","ref":"ProofOfReserves.Util.html#int_to_little/2"},{"type":"function","title":"ProofOfReserves.Util.is_power_of_two?/1","doc":"is_power_of_two? returns true if the given number is a power of two.","ref":"ProofOfReserves.Util.html#is_power_of_two?/1"},{"type":"function","title":"ProofOfReserves.Util.leaf_hash/3","doc":"leaf_hash calculates the hash of a leaf in the Merkle Sum Tree.","ref":"ProofOfReserves.Util.html#leaf_hash/3"},{"type":"function","title":"ProofOfReserves.Util.next_power_of_two/1","doc":"next_power_of_two returns the closest (larger) power of two to the given number.\nNote: this function returns the next power of two, not the closest.","ref":"ProofOfReserves.Util.html#next_power_of_two/1"},{"type":"function","title":"ProofOfReserves.Util.pad/3","doc":"pads binary according to the byte length and the padding type. A binary can be padded with leading or trailing zeros.","ref":"ProofOfReserves.Util.html#pad/3"},{"type":"function","title":"ProofOfReserves.Util.sats_to_btc/1","doc":"","ref":"ProofOfReserves.Util.html#sats_to_btc/1"},{"type":"function","title":"ProofOfReserves.Util.sha256/1","doc":"sha256 calculates the sha256 hash of the given data.","ref":"ProofOfReserves.Util.html#sha256/1"},{"type":"function","title":"ProofOfReserves.Util.sha256hmac/2","doc":"sha256hmac calculates the sha256 hmac of the given key and message.","ref":"ProofOfReserves.Util.html#sha256hmac/2"},{"type":"function","title":"ProofOfReserves.Util.str_to_int/1","doc":"str_to_int converts a string to an integer.","ref":"ProofOfReserves.Util.html#str_to_int/1"},{"type":"type","title":"ProofOfReserves.Util.pad_type/0","doc":"The pad_type describes the padding to use.","ref":"ProofOfReserves.Util.html#t:pad_type/0"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}